using DreamsCaster.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Console = System.Console;

namespace DreamsCaster.Controllers;

[ApiController]
[Route("[controller]")]
public class GameController : ControllerBase
{
	public AppDbContext Database { get; set; }

	public GameController(AppDbContext dbContext)
	{
		Database = dbContext;
	}

	#region Game CRUD

	[HttpGet("{gameId:guid}")]
	public IActionResult Get([FromRoute] Guid gameId)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		
		return Ok(game);
	}

	[HttpDelete("{gameId:guid}")]
	//[Authorize]
	public async Task<IActionResult> DeleteAsync([FromRoute] Guid gameId)
	{
		if (!Database.Games.TryGet(gameId, out var game)) return NotFound();

		foreach (var release in game.Releases)
		{
			Database.Remove(release);
		}

		await Database.SaveChangesAsync();

		Database.Games.Remove(game);
		
		await Database.SaveChangesAsync();

		return Ok();
	}

	[HttpPatch("{gameId:guid}")]
	//[Authorize]
	public async Task<IActionResult> PatchAsync([FromRoute] Guid gameId, [FromBody] PatchInfo newData)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();

		if (newData.Name.HasValue) game.Name = newData.Name;
		if (newData.Description.HasValue) game.Data.Description = newData.Description;
		if (newData.CoverPhoto.HasValue) game.Data.CoverPhoto = newData.CoverPhoto;
		
		await Database.SaveChangesAsync();

		return Ok(game);
	}

	[HttpPost]
	//[Authorize]
	public async Task<IActionResult> PostAsync([FromBody] PostGame newGame)
	{
		try
		{
			var game = Database.Games.Add(new Game()
			{
				Name = newGame.Name,
				Data = new MetaData<Game>()
				{
					CoverPhoto = newGame.CoverPhoto,
					Description = newGame.Description
				}
			});

			await Database.SaveChangesAsync();

			foreach (var release in newGame.Releases)
			{
				if (!Database.Consoles.TryGet(release.ConsoleId, out var console)) continue;

				game.Entity.Releases.Add(new Release()
				{
					ReleaseDate = release.Release,
					Console = console
				});
			}

			await Database.SaveChangesAsync();

			return Ok(game);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	#endregion Game CRUD
	
	#region Releases CUD

	[HttpPost("{gameId:guid}/releases")]
	//[Authorize]
	public async Task<IActionResult> PostReleaseAsync([FromRoute] Guid gameId, [FromBody] PostRelease release)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!Database.Consoles.TryGet(release.ConsoleId, out var console)) return NotFound();

		try
		{
			game.Releases.Add(Release.New(release.Release, console));

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	[HttpPut("{gameId:guid}/releases/{releaseId:guid}")]
	//[Authorize]
	public async Task<IActionResult> PatchReleaseAsync([FromRoute] Guid gameId, [FromRoute] Guid releaseId, [FromBody] DateTime newReleaseDate)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!game.Releases.TryGet(releaseId, out var release)) return NotFound();

		try
		{
			release.ReleaseDate = newReleaseDate;

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpDelete("{gameId:guid}/releases/{releaseId:guid}")]
	//[Authorize]
	public async Task<IActionResult> DeleteReleaseAsync([FromRoute] Guid gameId, [FromRoute] Guid releaseId)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!game.Releases.TryGet(releaseId, out Release release)) return NotFound();

		try
		{
			game.Releases.Remove(release);
			await Database.SaveChangesAsync();
			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	#endregion Releases CUD


	#region Developer AD


	[HttpPut("{gameId:guid}/developers/{developerId:guid}")]
	//[Authorize]
	public async Task<IActionResult> AddDeveloperAsync([FromRoute] Guid gameId, [FromRoute] Guid developerId)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!Database.Developers.TryGet(developerId, out var dev)) return NotFound();

		try
		{
			game.Developers.Add(dev);

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpDelete("{gameId:guid}/developers/{developerId:guid}")]
	//[Authorize]
	public async Task<IActionResult> RemoveDeveloperAsync([FromRoute] Guid gameId, [FromRoute] Guid developerId)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!game.Developers.TryGet(developerId, out var dev)) return NotFound();

		try
		{
			game.Developers.Remove(dev);

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	#endregion Developer AD


	#region Publisher AD
	
	[HttpPut("{gameId:guid}/publishers/{publisherId:guid}")]
	//[Authorize]
	public async Task<IActionResult> AddPublisherAsync([FromRoute] Guid gameId, [FromRoute] Guid publisherId)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!Database.Publishers.TryGet(publisherId, out var pub)) return NotFound();

		try
		{
			game.Publishers.Add(pub);

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}

	[HttpDelete("{gameId:guid}/publishers/{publisherId:guid}")]
	//[Authorize]
	public async Task<IActionResult> RemovePublisherAsync([FromRoute] Guid gameId, [FromRoute] Guid publisherId)
	{
		if (!Database.Games.TryGet(gameId, out Game game)) return NotFound();
		if (!game.Publishers.TryGet(publisherId, out Publisher release)) return NotFound();

		try
		{
			game.Publishers.Remove(release);

			await Database.SaveChangesAsync();

			return Ok();
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
	
	#endregion Publisher AD
}